@@ -450,6 +450,17 @@ class NiryoRobot(object):
         pose_list = self.__args_pose_to_list(*args)
         self.__send_n_receive(Command.MOVE_POSE, *pose_list)
 
+    def move_linear_pose(self, *args):
+        """
+        Move robot end effector pose to a (x, y, z, roll, pitch, yaw) pose with a linear trajectory
+
+        :param args: either 6 args (1 for each coordinates) or a list of 6 coordinates or a PoseObject
+        :type args: Union[tuple[float], list[float], PoseObject]
+        :rtype: None
+        """
+        pose_list = self.__args_pose_to_list(*args)
+        self.__send_n_receive(Command.MOVE_LINEAR_POSE, *pose_list)
+
     def shift_pose(self, axis, shift_value):
         """
         Shift robot end effector pose along one axis
@@ -461,10 +472,23 @@ class NiryoRobot(object):
         :rtype: None
         """
         self.__check_enum_belonging(axis, RobotAxis)
-
         shift_value = self.__transform_to_type(shift_value, float)
         self.__send_n_receive(Command.SHIFT_POSE, axis, shift_value)
 
+    def shift_linear_pose(self, axis, shift_value):
+        """
+        Shift robot end effector pose along one axis, with a linear trajectory
+
+        :param axis: Axis along which the robot is shifted
+        :type axis: RobotAxis
+        :param shift_value: In meter for X/Y/Z and radians for roll/pitch/yaw
+        :type shift_value: float
+        :rtype: None
+        """
+        self.__check_enum_belonging(axis, RobotAxis)
+        shift_value = self.__transform_to_type(shift_value, float)
+        self.__send_n_receive(Command.SHIFT_LINEAR_POSE, axis, shift_value)
+
     def jog_joints(self, *args):
         """
         Jog robot joints'.
@@ -492,17 +516,6 @@ class NiryoRobot(object):
         pose_offset = self.__args_joints_to_list(*args)
         self.__send_n_receive(Command.JOG_POSE, *pose_offset)
 
-    def move_linear_pose(self, *args):
-        """
-        Move robot end effector pose to a (x, y, z, roll, pitch, yaw) pose in a linear way
-
-        :param args: either 6 args (1 for each coordinates) or a list of 6 coordinates or a PoseObject
-        :type args: Union[tuple[float], list[float], PoseObject]
-        :rtype: None
-        """
-        pose_list = self.__args_pose_to_list(*args)
-        self.__send_n_receive(Command.MOVE_LINEAR_POSE, *pose_list)
-
     def move_to_home_pose(self):
         """
         Move to a position where the forearm lays on shoulder
@@ -677,34 +690,61 @@ class NiryoRobot(object):
         """
         Execute trajectory from list of poses
 
-        :param list_poses: List of [x,y,z,qx,qy,qz,qw]
+        :param list_poses: List of [x,y,z,qx,qy,qz,qw] or list of [x,y,z,roll,pitch,yaw]
         :type list_poses: list[list[float]]
         :param dist_smoothing: Distance from waypoints before smoothing trajectory
         :type dist_smoothing: float
         :rtype: None
         """
         for i, pose in enumerate(list_poses):
-            if len(pose) != 7:
-                self.__raise_exception("7 parameters expected in a pose [x,y,z,qx,qy,qz,qw], "
-                                       "{} parameters given".format(len(pose)))
+            if len(pose) != 7 and len(pose) != 6:
+                self.__raise_exception(
+                    "7 parameters expected in a pose [x,y,z,qx,qy,qz,qw], or 6 in a pose [x,y,z,roll,pitch,yaw], "
+                    "{} parameters given".format(len(pose)))
             list_poses[i] = self.__map_list(pose, float)
 
         self.__send_n_receive(Command.EXECUTE_TRAJECTORY_FROM_POSES, list_poses, dist_smoothing)
 
+    def execute_trajectory_from_poses_and_joints(self, list_pose_joints, list_type=None, dist_smoothing=0.0):
+        """
+        Execute trajectory from list of poses and joints
+
+        :param list_pose_joints: List of [x,y,z,qx,qy,qz,qw] or list of [x,y,z,roll,pitch,yaw] or a list of [j1,j2,j3,j4,j5,j6]
+        :type list_pose_joints: list[list[float]]
+        :param list_type: List of string 'pose' or 'joint', or ['pose'] (if poses only) or ['joint'] (if joints only). 
+                        If None, it is assumed there are only poses in the list.
+        :type list_type: list[string]
+        :param dist_smoothing: Distance from waypoints before smoothing trajectory
+        :type dist_smoothing: float
+        :rtype: None
+        """
+        if list_type is None:
+            list_type = ['pose']
+        for i, pose_or_joint in enumerate(list_pose_joints):
+            if len(pose_or_joint) != 7 and len(pose_or_joint) != 6:
+                self.__raise_exception(
+                    "7 parameters expected in a pose [x,y,z,qx,qy,qz,qw], or 6 in a pose [x,y,z,roll,pitch,yaw], "
+                    "or 6 in a joint [j1,j2,j3,j4,j5,j6]"
+                    "{} parameters given".format(len(pose_or_joint)))
+            list_pose_joints[i] = self.__map_list(pose_or_joint, float)
+        self.__send_n_receive(Command.EXECUTE_TRAJECTORY_FROM_POSES_AND_JOINTS, list_pose_joints, list_type,
+                              dist_smoothing)
+
     def save_trajectory(self, trajectory_name, list_poses):
         """
         Save trajectory in robot memory
 
         :type trajectory_name: str
-        :param list_poses: List of [x,y,z,qx,qy,qz,qw]
+        :param list_poses: List of [x,y,z,qx,qy,qz,qw] or list of [x,y,z,roll,pitch,yaw]
         :type list_poses: list[list[float]]
         :rtype: None
         """
         self.__check_type(trajectory_name, str)
         for i, pose in enumerate(list_poses):
-            if len(pose) != 7:
-                self.__raise_exception("7 parameters expected in a pose [x,y,z,qx,qy,qz,qw], "
-                                       "{} parameters given".format(len(pose)))
+            if len(pose) != 7 and len(pose) != 6:
+                self.__raise_exception(
+                    "7 parameters expected in a pose [x,y,z,qx,qy,qz,qw], or 6 in a pose [x,y,z,roll,pitch,yaw], "
+                    "{} parameters given".format(len(pose)))
             list_poses[i] = self.__map_list(pose, float)
 
         self.__send_n_receive(Command.SAVE_TRAJECTORY, trajectory_name, list_poses)
@@ -850,9 +890,51 @@ class NiryoRobot(object):
         :rtype: None
         """
         self.__check_enum_belonging(pin_id, PinID)
-
         self.__send_n_receive(Command.DEACTIVATE_ELECTROMAGNET, pin_id)
 
+    def enable_tcp(self, enable=True):
+        """
+        Enables or disables the TCP function (Tool Center Point).
+        If activation is requested, the last recorded TCP value will be applied.
+        The default value depends on the gripper equipped.
+        If deactivation is requested, the TCP will be coincident with the tool_link.
+
+        :param enable: True to enable, False otherwise.
+        :type enable: Bool
+        :rtype: None
+        """
+        self.__check_instance(enable, bool)
+        self.__send_n_receive(Command.ENABLE_TCP, enable)
+
+    def set_tcp(self, *args):
+        """
+        Activates the TCP function (Tool Center Point)
+        and defines the transformation between the tool_link frame and the TCP frame.
+
+        :param args: either 6 args (1 for each coordinates) or a list of 6 coordinates or a PoseObject
+        :type args: Union[list[float], tuple[float], PoseObject]
+        :rtype: None
+        """
+        tcp_transform = self.__args_pose_to_list(*args)
+        self.__send_n_receive(Command.SET_TCP, *tcp_transform)
+
+    def reset_tcp(self):
+        """
+        Reset the TCP (Tool Center Point) transformation.
+        The TCP will be reset according to the tool equipped.
+
+        :rtype: None
+        """
+        self.__send_n_receive(Command.RESET_TCP)
+
+    def tool_reboot(self):
+        """
+        Reboot the motor of the tool equipped. Useful when an Overload error occurs. (cf HardwareStatus)
+
+        :rtype: None
+        """
+        self.__send_n_receive(Command.TOOL_REBOOT)
+
     # - Hardware
 
     def set_pin_mode(self, pin_id, pin_mode):
@@ -954,7 +1036,7 @@ class NiryoRobot(object):
         """
         Activate a new conveyor and return its ID
 
-        :return : New conveyor ID
+        :return: New conveyor ID
         :rtype: ConveyorID
         """
         conveyor_id_str = self.__send_n_receive(Command.SET_CONVEYOR)
@@ -1021,7 +1103,7 @@ class NiryoRobot(object):
         """
         self.__check_enum_belonging(conveyor_id, ConveyorID)
         self.__check_type(control_on, bool)
-        self.__transform_to_type(speed, bool)
+        self.__transform_to_type(speed, int)
         self.__check_enum_belonging(direction, ConveyorDirection)
 
         self.__send_n_receive(Command.CONTROL_CONVEYOR, conveyor_id, control_on, speed, direction)
@@ -1049,6 +1131,66 @@ class NiryoRobot(object):
         _, img = self.__send_n_receive(Command.GET_IMAGE_COMPRESSED, with_payload=True)
         return img
 
+    def set_brightness(self, brightness_factor):
+        """
+        Modify video stream brightness
+
+        :param brightness_factor: How much to adjust the brightness. 0.5 will
+            give a darkened image, 1 will give the original image while
+            2 will enhance the brightness by a factor of 2.
+        :type brightness_factor: float
+        :rtype: None
+        """
+        self.__transform_to_type(brightness_factor, float)
+        self.__check_range_belonging(brightness_factor, 0.0, np.Inf)
+        self.__send_n_receive(Command.SET_IMAGE_BRIGHTNESS, brightness_factor)
+
+    def set_contrast(self, contrast_factor):
+        """
+        Modify video stream contrast
+
+        :param contrast_factor: While a factor of 1 gives original image.
+            Making the factor towards 0 makes the image greyer, while factor>1 increases the contrast of the image.
+        :type contrast_factor: float
+        :rtype: None
+        """
+        self.__transform_to_type(contrast_factor, float)
+        self.__check_range_belonging(contrast_factor, 0.0, np.Inf)
+        self.__send_n_receive(Command.SET_IMAGE_CONTRAST, contrast_factor)
+
+    def set_saturation(self, saturation_factor):
+        """
+        Modify video stream saturation
+
+        :param saturation_factor: How much to adjust the saturation. 0 will
+            give a black and white image, 1 will give the original image while
+            2 will enhance the saturation by a factor of 2.
+        :type saturation_factor: float
+        :rtype: None
+        """
+        self.__transform_to_type(saturation_factor, float)
+        self.__check_range_belonging(saturation_factor, 0.0, np.Inf)
+        self.__send_n_receive(Command.SET_IMAGE_SATURATION, saturation_factor)
+
+    def get_image_parameters(self):
+        """
+        Get last stream image parameters: Brightness factor, Contrast factor, Saturation factor.
+
+        Brightness factor: How much to adjust the brightness. 0.5 will give a darkened image,
+        1 will give the original image while 2 will enhance the brightness by a factor of 2.
+
+        Contrast factor: A factor of 1 gives original image.
+        Making the factor towards 0 makes the image greyer, while factor>1 increases the contrast of the image.
+
+        Saturation factor: 0 will give a black and white image, 1 will give the original image while
+        2 will enhance the saturation by a factor of 2.
+
+        :return:  Brightness factor, Contrast factor, Saturation factor
+        :rtype: float, float, float
+        """
+        brightness_factor, contrast_factor, saturation_factor = self.__send_n_receive(Command.GET_IMAGE_PARAMETERS)
+        return brightness_factor, contrast_factor, saturation_factor
+
     def get_target_pose_from_rel(self, workspace_name, height_offset, x_rel, y_rel, yaw_rel):
         """
         Given a pose (x_rel, y_rel, yaw_rel) relative to a workspace, this function
@@ -1234,7 +1376,7 @@ class NiryoRobot(object):
 
         Poses should be either a list [x, y, z, roll, pitch, yaw] or a PoseObject
 
-        :param workspace_name: workspace name
+        :param workspace_name: workspace name, maximum lenght 30 char.
         :type workspace_name: str
         :param pose_origin:
         :type pose_origin: Union[list[float], PoseObject]
@@ -1260,7 +1402,7 @@ class NiryoRobot(object):
         Save workspace by giving the points of worskpace's 4 corners. Points are written as [x, y, z]
         Corners should be in the good order.
 
-        :param workspace_name: workspace name
+        :param workspace_name: workspace name, maximum lenght 30 char.
         :type workspace_name: str
         :param point_origin:
         :type point_origin: list[float]
